      SUBROUTINE RICCND (NF,NG,NH,NZ,N,NN,F,G,H,Z,W,ER,EI,WORK,
     +                   SCALE,ITYPE,IPVL,IPVS)
C
C     *****PARAMETERS:
      INTEGER NF,NG,NH,NZ,N,NN,ITYPE(NN),IPVL(NN),IPVS(N)
      DOUBLE PRECISION F(NF,N),G(NG,N),H(NH,N),Z(NZ,NN),W(NZ,NN),
     +                 ER(NN),EI(NN),WORK(N),SCALE(NN)
C
C     *****LOCAL VARIABLES:
      INTEGER I,J,LOW,IGH,NLOW,NPI,NPJ,NUP
      DOUBLE PRECISION EPS,EPSP1,ZNORM,T,ALPHA,COND
C
C     *****FUNCTIONS:
      DOUBLE PRECISION DABS,DSQRT
C
C     *****SUBROUTINES CALLED:
C     HQR3,MLINEQ
C     BALANC,BALBAK,ORTHES,ORTRAN (EISPACK)
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     THIS SUBROUTINE SOLVES THE CONTINUOUS-TIME ALGEBRAIC
C     MATRIX RICCATI EQUATION
C                      T
C                     F *X + X*F - X*G*X + H = 0
C
C     BY LAUB'S VARIANT OF THE HAMILTONIAN-EIGENVECTOR APPROACH.
C     REF.: LAUB, A.J., A SCHUR METHOD FOR SOLVING ALGEBRAIC RICCATI
C           EQUATIONS, IEEE TRANS. AUT. CONTR., AC-24(1979), 913-921.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NF,NG,NH,NZ      ROW DIMENSIONS OF THE ARRAYS CONTAINING
C                         F,G,H,Z (AND W), RESPECTIVELY, AS
C                         DECLARED IN THE CALLING PROGRAM DIMENSION
C                         STATEMENT;
C
C        N                ORDER OF THE MATRICES F,G,H;
C
C        NN               = 2*N = ORDER OF THE INTERNALLY GENERATED
C                         MATRICES Z AND W;
C
C        F                N X N (REAL) MATRIX;
C
C        G,H              N X N SYMMETRIC, NONNEGATIVE DEFINITE
C                         (REAL) MATRICES.
C
C     ON OUTPUT:
C
C        H                N X N ARRAY CONTAINING THE UNIQUE POSITIVE
C                         (OR NONNEGATIVE) DEFINITE SOLUTION OF THE
C                         RICCATI EQUATION;
C
C        ER,EI            REAL SCRATCH VECTORS OF LENGTH 2*N; ON OUTPUT
C                         (ER(I),EI(I)), I=1,N CONTAIN THE REAL AND
C                         IMAGINARY PARTS, RESPECTIVELY, OF THE N
C                         CLOSED LOOP EIGENVALUES (I.E., THE
C                         SPECTRUM OF F - G*X);
C
C        Z,W              2*N X 2*N REAL SCRATCH ARRAYS USED FOR
C                         COMPUTATIONS INVOLVING THE HAMILTONIAN
C                         MATRIX ASSOCIATED WITH THE RICCATI EQUATION;
C
C        WORK,SCALE       REAL SCRATCH VECTORS OF LENGTHS N, 2*N,
C                         RESPECTIVELY;  ON OUTPUT, WORK(1) CONTAINS A
C                         CONDITION NUMBER ESTIMATE FOR THE FINAL NTH
C                         ORDER LINEAR MATRIX EQUATION SOLVED;
C
C        ITYPE,IPVL,IPVS  INTEGER SCRATCH VECTORS OF LENGTHS 2*N, 2*N,
C                         N, RESPECTIVELY.
C
C     ***NOTE:  ALL SCRATCH ARRAYS MUST BE DECLARED AND INCLUDED
C               IN THE CALL.***
C
C     *****ALGORITHM NOTES:
C     IT IS ASSUMED THAT G AND H ARE NONNEGATIVE DEFINITE AND THAT (F,B)
C                                                      T
C     IS STABILIZABLE AND (C,F) IS DETECTABLE WHERE B*B  = G
C                                       T
C     (B OF FULL RANK = RANK(G)) AND C*C = H (C OF FULL
C     RANK = RANK(H)) IN WHICH CASE THE SOLUTION (RETURNED IN THE
C     ARRAY H) IS UNIQUE AND NONNEGATIVE DEFINITE.
C
C     *****HISTORY:
C     WRITTEN BY ALAN J. LAUB (PRESENTLY AT: ECE DEPT., UNIV. OF
C     CALIFORNIA, SANTA BARBARA, CA 93106; PH.: (805) 961-3616),
C     SEP. 1977.
C     MOST RECENT VERSION:  APR. 15, 1981.
C
C     ------------------------------------------------------------------
C
C     EPS IS AN INTERNALLY GENERATED MACHINE DEPENDENT PARAMETER
C     SPECIFYING THE RELATIVE PRECISION OF FLOATING POINT ARITHMETIC.
C     FOR EXAMPLE, EPS = 16.0D0**(-13) FOR DOUBLE PRECISION ARITHMETIC
C     ON IBM S360/S370.
C
      EPS=1.0D0
    5 EPS=EPS/2.0D0
      EPSP1=EPS+1.0D0
      IF (EPSP1.GT.1.0D0) GO TO 5
      EPS=2.0D0*EPS
C
C     SET UP HAMILTONIAN MATRIX
C
      DO 20 J=1,N
         NPJ=N+J
         DO 10 I=1,N
            NPI=N+I
            Z(I,J)=F(I,J)
            Z(NPI,J)=-H(I,J)
            Z(I,NPJ)=-G(I,J)
            Z(NPI,NPJ)=-F(J,I)
   10    CONTINUE
   20 CONTINUE
C
C     BALANCE Z
C
      CALL BALANC (NZ,NN,Z,LOW,IGH,SCALE)
C
C     COMPUTE 1-NORM OF Z
C
      ZNORM=0.0D0
      DO 40 J=1,NN
         T=0.0D0
         DO 30 I=1,NN
            T=T+DABS(Z(I,J))
   30    CONTINUE
         IF (T.GT.ZNORM) ZNORM=T
   40 CONTINUE
      ALPHA=DSQRT(ZNORM)+1.0D0
C
C                              -1
C     COMPUTE W = (ALPHA*I + Z)  *(ALPHA*I - Z), AN ANALYTIC FUNCTION
C     OF Z MAPPING THE LEFT HALF PLANE TO THE EXTERIOR OF THE UNIT
C     DISK.  THIS PERMITS DIRECT APPLICATION OF HQR3.  THIS STEP MAY
C     BE REMOVED IF HQR3 IS MODIFIED APPROPRIATELY.
C
      DO 60 J=1,NN
         DO 50 I=1,NN
            W(I,J)=-Z(I,J)
   50    CONTINUE
         W(J,J)=ALPHA+W(J,J)
         Z(J,J)=ALPHA+Z(J,J)
   60 CONTINUE
      CALL MLINEQ (NZ,NZ,NN,NN,Z,W,COND,IPVL,ER)
C
C     REDUCE W TO REAL SCHUR FORM WITH EIGENVALUES OUTSIDE THE UNIT
C     DISK IN THE UPPER LEFT N X N UPPER QUASI-TRIANGULAR BLOCK
C
      NLOW=1
      NUP=NN
      CALL ORTHES (NZ,NN,NLOW,NUP,W,ER)
      CALL ORTRAN (NZ,NN,NLOW,NUP,W,ER,Z)
      CALL HQR3 (W,Z,NN,NLOW,NUP,EPS,ER,EI,ITYPE,NZ,NZ)
C
C     COMPUTE SOLUTION OF THE RICCATI EQUATION FROM THE ORTHOGONAL
C     MATRIX NOW IN THE ARRAY Z.  STORE THE RESULT IN THE ARRAY H.
C
      CALL BALBAK (NZ,NN,LOW,IGH,SCALE,NN,Z)
      DO 80 J=1,N
         NPJ=N+J
         DO 70 I=1,N
            F(I,J)=Z(J,I)
            H(I,J)=Z(NPJ,I)
   70    CONTINUE
   80 CONTINUE
      CALL MLINEQ (NF,NH,N,N,F,H,COND,IPVS,WORK)
      WORK(1)=COND
C
C     TRANSFORM BACK TO GET THE CLOSED-LOOP SPECTRUM
C
      DO 110 I=1,N
         IF (ITYPE(I).GE.0) GO TO 90
         WRITE (5,44400) I
44400    FORMAT (1X,I4,1X,41HTH EIGENVALUE NOT SUCCESSFULLY CALCULATED)
         RETURN
   90    IF (ITYPE(I).GT.0) GO TO 100
         ER(I)=ALPHA*(1.0D0-ER(I))/(1.0D0+ER(I))
         EI(I)=0.0D0
         GO TO 110
  100    IF (ITYPE(I).EQ.2) GO TO 110
         T=ALPHA/((1.0D0+ER(I))**2+EI(I)**2)
         ER(I)=(1.0D0-ER(I)**2-EI(I)**2)*T
         EI(I)=-2.0D0*EI(I)*T
         ER(I+1)=ER(I)
         EI(I+1)=-EI(I)
  110 CONTINUE
      RETURN
C
C     LAST LINE OF RICCND
C
      END

      SUBROUTINE HQR3 (A,V,N,NLOW,NUP,EPS,ER,EI,ITYPE,NA,NV)
C
C     *****PARAMETERS:
      INTEGER N,NA,NLOW,NUP,NV,ITYPE(N)
      DOUBLE PRECISION A(NA,N),EI(N),ER(N),EPS,V(NV,N)
C
C     *****LOCAL VARIABLES:
      LOGICAL FAIL
      INTEGER I,IT,L,MU,NL,NU
      DOUBLE PRECISION E1,E2,P,Q,R,S,T,W,X,Y,Z
C
C     *****FUNCTIONS:
      DOUBLE PRECISION DABS
C
C     *****SUBROUTINES CALLED:
C     EXCHNG,QRSTEP,SPLIT
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     THIS SUBROUTINE REDUCES THE UPPER HESSENBERG MATRIX A TO QUASI-
C     TRIANGULAR FORM BY UNITARY SIMILARITY TRANSFORMATIONS. THE
C     EIGENVALUES OF A, WHICH ARE CONTAINED IN THE 1 X 1 AND 2 X 2
C     DIAGONAL BLOCKS OF THE REDUCED MATRIX, ARE ORDERED IN DESCENDING
C     ORDER OF MAGNITUDE ALONG THE DIAGONAL. THE TRANSFORMATIONS ARE
C     ACCUMULATED IN THE ARRAY V.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NA,NV            ROW DIMENSIONS OF THE ARRAYS CONTAINING A AND
C                         V, RESPECTIVELY, AS DECLARED IN THE CALLING
C                         PROGRAM DIMENSION STATEMENT;
C
C        A                N X N ARRAY CONTAINING THE UPPER HESSENBERG
C                         MATRIX TO BE REDUCED;
C
C        N                ORDER OF THE MATRICES A AND V;
C
C        NLOW,NUP         A(NLOW,NLOW-1) AND A(NUP,1+NUP) ARE ASSUMED
C                         TO BE ZERO, AND ONLY ROWS NLOW THROUGH NUP
C                         AND COLUMNS NLOW THROUGH NUP ARE TRANSFORMED,
C                         RESULTING IN THE CALCULATION OF EIGENVALUES
C                         NLOW THROUGH NUP;
C
C        EPS              A CONVERGENCE CRITERION USED TO DETERMINE WHEN
C                         A SUBDIAGONAL ELEMENT OF A IS NEGLIGIBLE.
C                         SPECIFICALLY, A(I+1,I) IS REGARDED AS
C                         NEGLIGIBLE IF DABS(A(I+1),I)).LE.EPS*
C                         (DABS(A(I+1,I+1))). THIS MEANS THAT THE FINAL
C                         MATRIX RETURNED BY THE PROGRAM WILL BE EXACTLY
C                         SIMILAR TO A + E WHERE E IS OF ORDER
C                         EPS*NORM(A), FOR ANY REASONABLY BALANCED NORM
C                         SUCH AS THE ROW-SUM NORM;
C
C        ITYPE            AN INTEGER VECTOR OF LENGTH N WHOSE
C                         I-TH ENTRY IS
C                         0  IF THE I-TH EIGENVALUE IS REAL,
C                         1  IF THE I-TH EIGENVALUE IS COMPLEX WITH
C                            POSITIVE IMAGINARY PART,
C                         2  IF THE I-TH EIGENVALUE IS COMPLEX WITH
C                            NEGATIVE IMAGINARY PART,
C                         -1 IF THE I-TH EIGENVALUE WAS NOT CALCULATED
C                            SUCCESSFULLY.
C
C        ON OUTPUT:
C
C        A                N X N ARRAY CONTAINING THE REDUCED, QUASI-
C                         TRIANGULAR MATRIX;
C
C        V                N X N ARRAY CONTAINING THE REDUCING
C                         TRANSFORMATIONS TO BE MULTIPLIED;
C
C        ER,EI            REAL SCRATCH VECTORS OF LENGTH N WHICH ON
C                         RETURN CONTAIN THE REAL AND IMAGINARY PARTS,
C                         RESPECTIVELY, OF THE EIGENVALUES.
C
C     *****HISTORY:
C              ADAPTED FROM STEWART'S HQR3 SOFTWARE.
C
C     ------------------------------------------------------------------
C
      DO 10 I=NLOW,NUP
         ITYPE(I)=-1
10    CONTINUE
      T=0.0D0
      NU=NUP
20    IF (NU.LT.NLOW) GO TO 240
      IT=0
30    CONTINUE
      L=NU
40    CONTINUE
      IF (L.EQ.NLOW) GO TO 50
      IF (DABS(A(L,L-1)).LE.EPS*(DABS(A(L-1,L-1))+DABS(A(L,L))))
     +    GO TO 50
      L=L-1
      GO TO 40
50    CONTINUE
      X=A(NU,NU)
      IF (L.EQ.NU) GO TO 160
      Y=A(NU-1,NU-1)
      W=A(NU,NU-1)*A(NU-1,NU)
      IF (L.EQ.NU-1) GO TO 100
      IF (IT.EQ.30) GO TO 240
      IF (IT.NE.10 .AND. IT.NE.20) GO TO 70
      T=T+X
      DO 60 I=NLOW,NU
         A(I,I)=A(I,I)-X
60    CONTINUE
      S=DABS(A(NU,NU-1))+DABS(A(NU-1,NU-2))
      X=0.75D0*S
      Y=X
      W=-0.4375D0*S**2
70    CONTINUE
      IT=IT+1
      NL=NU-2
80    CONTINUE
      Z=A(NL,NL)
      R=X-Z
      S=Y-Z
      P=(R*S-W)/A(NL+1,NL)+A(NL,NL+1)
      Q=A(NL+1,NL+1)-Z-R-S
      R=A(NL+2,NL+1)
      S=DABS(P)+DABS(Q)+DABS(R)
      P=P/S
      Q=Q/S
      R=R/S
      IF (NL.EQ.L) GO TO 90
      IF (DABS(A(NL,NL-1))*(DABS(Q)+DABS(R)).LE.EPS*DABS(P)*
     +    (DABS(A(NL-1,NL-1))+DABS(Z)+DABS(A(NL+1,NL+1)))) GO TO 90
      NL=NL-1
      GO TO 80
90    CONTINUE
      CALL QRSTEP (A,V,P,Q,R,NL,NU,N,NA,NV)
      GO TO 30
100   IF (NU.NE.NLOW+1) A(NU-1,NU-2)=0.0D0
      A(NU,NU)=A(NU,NU)+T
      A(NU-1,NU-1)=A(NU-1,NU-1)+T
      ITYPE(NU)=0
      ITYPE(NU-1)=0
      MU=NU
110   CONTINUE
      NL=MU-1
      CALL SPLIT (A,V,N,NL,E1,E2,NA,NV)
      IF (A(MU,MU-1).EQ.0.0D0) GO TO 170
      IF (MU.EQ.NUP) GO TO 230
      IF (MU.EQ.NUP-1) GO TO 130
      IF (A(MU+2,MU+1).EQ.0.0D0) GO TO 130
      IF (A(MU-1,MU-1)*A(MU,MU)-A(MU-1,MU)*A(MU,MU-1).GE.A(MU+1,MU+1)*
     +    A(MU+2,MU+2)-A(MU+1,MU+2)*A(MU+2,MU+1)) GO TO 230
      CALL EXCHNG (A,V,N,NL,2,2,EPS,FAIL,NA,NV)
      IF (.NOT.FAIL) GO TO 120
      ITYPE(NL)=-1
      ITYPE(NL+1)=-1
      ITYPE(NL+2)=-1
      ITYPE(NL+3)=-1
      GO TO 240
120   CONTINUE
      MU=MU+2
      GO TO 150
130   CONTINUE
      IF (A(MU-1,MU-1)*A(MU,MU)-A(MU-1,MU)*A(MU,MU-1).GE.
     +    A(MU+1,MU+1)**2) GO TO 230
      CALL EXCHNG (A,V,N,NL,2,1,EPS,FAIL,NA,NV)
      IF (.NOT.FAIL) GO TO 140
      ITYPE(NL)=-1
      ITYPE(NL+1)=-1
      ITYPE(NL+2)=-1
      GO TO 240
140   CONTINUE
      MU=MU+1
150   CONTINUE
      GO TO 110
160   NL=0
      A(NU,NU)=A(NU,NU)+T
      IF (NU.NE.NLOW) A(NU,NU-1)=0.0D0
      ITYPE(NU)=0
      MU=NU
170   CONTINUE
180   CONTINUE
      IF (MU.EQ.NUP) GO TO 220
      IF (MU.EQ.NUP-1) GO TO 200
      IF (A(MU+2,MU+1).EQ.0.0D0) GO TO 200
      IF (A(MU,MU)**2.GE.A(MU+1,MU+1)*A(MU+2,MU+2)-A(MU+1,MU+2)*
     +    A(MU+2,MU+1)) GO TO 220
      CALL EXCHNG (A,V,N,MU,1,2,EPS,FAIL,NA,NV)
      IF (.NOT.FAIL) GO TO 190
      ITYPE(MU)=-1
      ITYPE(MU+1)=-1
      ITYPE(MU+2)=-1
      GO TO 240
190   CONTINUE
      MU=MU+2
      GO TO 210
200   CONTINUE
      IF (DABS(A(MU,MU)).GE.DABS(A(MU+1,MU+1))) GO TO 220
      CALL EXCHNG (A,V,N,MU,1,1,EPS,FAIL,NA,NV)
      MU=MU+1
210   CONTINUE
      GO TO 180
220   CONTINUE
      MU=NL
      NL=0
      IF (MU.NE.0) GO TO 170
230   CONTINUE
      NU=L-1
      GO TO 20
240   IF (NU.LT.NLOW) GO TO 260
      DO 250 I=NLOW,NU
         A(I,I)=A(I,I)+T
250   CONTINUE
260   CONTINUE
      NU=NUP
270   CONTINUE
      IF (ITYPE(NU).NE.(-1)) GO TO 280
      NU=NU-1
      GO TO 310
280   CONTINUE
      IF (NU.EQ.NLOW) GO TO 290
      IF (A(NU,NU-1).EQ.0.0D0) GO TO 290
      CALL SPLIT (A,V,N,NU-1,E1,E2,NA,NV)
      IF (A(NU,NU-1).EQ.0.0D0) GO TO 290
      ER(NU)=E1
      EI(NU-1)=E2
      ER(NU-1)=ER(NU)
      EI(NU)=-EI(NU-1)
      ITYPE(NU-1)=1
      ITYPE(NU)=2
      NU=NU-2
      GO TO 300
290   CONTINUE
      ER(NU)=A(NU,NU)
      EI(NU)=0.0D0
      NU=NU-1
300   CONTINUE
310   CONTINUE
      IF (NU.GE.NLOW) GO TO 270
      RETURN
C
C     LAST LINE OF HQR3
C
      END
      SUBROUTINE EXCHNG (A,V,N,L,B1,B2,EPS,FAIL,NA,NV)
C
C     *****PARAMETERS
      INTEGER N,B1,B2,L,NA,NV
      DOUBLE PRECISION A(NA,N),EPS,V(NV,N)
      LOGICAL FAIL
C
C     *****LOCAL VARIABLES:
      INTEGER I,IT,J,L1,M
      DOUBLE PRECISION P,Q,R,S,W,X,Y,Z
C
C     *****FUNCTIONS:
      DOUBLE PRECISION DABS,DSQRT,DMAX1
C
C     *****SUBROUTINES CALLED:
C     QRSTEP
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     GIVEN THE UPPER HESSENBERG MATRIX A WITH CONSECUTIVE B1 X B1 AND
C     B2 X B2 DIAGONAL BLOCKS (B1, B2.LE.2) STARTING AT A(L,L), THIS
C     SUBROUTINE PRODUCES A UNITARY SIMILARITY TRANSFORMATION THAT
C     EXCHANGES THE BLOCKS ALONG WITH THEIR EIGENVALUES. THE
C     TRANSFORMATION IS ACCUMULATED IN V.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NA,NV            ROW DIMENSIONS OF THE ARRAYS CONTAINING A
C                         AND V, RESPECTIVELY, AS DECLARED IN THE
C                         CALLING PROGRAM DIMENSION STATEMENT;
C
C        A                N X N MATRIX WHOSE BLOCKS ARE TO BE
C                         INTERCHANGED;
C
C        N                ORDER OF THE MATRIX A;
C
C        L                POSITION OF THE BLOCKS;
C
C        B1               AN INTEGER CONTAINING THE SIZE OF THE FIRST
C                         BLOCK;
C
C        B2               AN INTEGER CONTAINING THE SIZE OF THE SECOND
C                         BLOCK;
C
C        EPS              A CONVERGENCE CRITERION (CF. HQR3).
C
C     ON OUTPUT:
C
C        FAIL             A LOGICAL VARIABLE WHICH IS .FALSE. ON A
C                         NORMAL RETURN. IF THIRTY ITERATIONS WERE
C                         PERFORMED WITHOUT CONVERGENCE, FAIL IS SET TO
C                         .TRUE. AND THE ELEMENT A(L+B2,L+B2-1) CANNOT
C                         BE ASSUMED ZERO.
C
C     *****HISTORY:
C              ADAPTED FROM STEWART'S HQR3 SOFTWARE.
C              HAMMARLING'S CORRECTION ADDED BY AJL 01/23/86
C
C     ------------------------------------------------------------------
C
      FAIL=.FALSE.
      IF (B1.EQ.2) GO TO 70
      IF (B2.EQ.2) GO TO 40
      L1=L+1
      Q=A(L+1,L+1)-A(L,L)
      IF (Q .EQ. 0.0D0) RETURN
      P=A(L,L+1)
      R=DMAX1(DABS(P),DABS(Q))
      IF (R.EQ.0.0D0) R = Q
      P=P/R
      Q=Q/R
      R=DSQRT(P**2+Q**2)
      P=P/R
      Q=Q/R
      DO 10 J=L,N
         S=P*A(L,J)+Q*A(L+1,J)
         A(L+1,J)=P*A(L+1,J)-Q*A(L,J)
         A(L,J)=S
10    CONTINUE
      DO 20 I=1,L1
         S=P*A(I,L)+Q*A(I,L+1)
         A(I,L+1)=P*A(I,L+1)-Q*A(I,L)
         A(I,L)=S
20    CONTINUE
      DO 30 I=1,N
         S=P*V(I,L)+Q*V(I,L+1)
         V(I,L+1)=P*V(I,L+1)-Q*V(I,L)
         V(I,L)=S
30    CONTINUE
      A(L+1,L)=0.0D0
      RETURN
40    CONTINUE
      X=A(L,L)
      P=1.0D0
      Q=1.0D0
      R=1.0D0
      CALL QRSTEP (A,V,P,Q,R,L,L+2,N,NA,NV)
      IT=0
50    IT=IT+1
      IF (IT.LE.30) GO TO 60
      FAIL=.TRUE.
      RETURN
60    CONTINUE
      P=A(L,L)-X
      Q=A(L+1,L)
      R=0.0D0
      CALL QRSTEP (A,V,P,Q,R,L,L+2,N,NA,NV)
      IF (DABS(A(L+2,L+1)).GT.EPS*(DABS(A(L+1,L+1))+DABS(A(L+2,L+2))))
     +    GO TO 50
      A(L+2,L+1)=0.0D0
      RETURN
70    CONTINUE
      M=L+2
      IF (B2.EQ.2) M=M+1
      X=A(L+1,L+1)
      Y=A(L,L)
      W=A(L+1,L)*A(L,L+1)
      P=1.0D0
      Q=1.0D0
      R=1.0D0
      CALL QRSTEP (A,V,P,Q,R,L,M,N,NA,NV)
      IT=0
80    IT=IT+1
      IF (IT.LE.30) GO TO 90
      FAIL=.TRUE.
      RETURN
90    CONTINUE
      Z=A(L,L)
      R=X-Z
      S=Y-Z
      P=(R*S-W)/A(L+1,L)+A(L,L+1)
      Q=A(L+1,L+1)-Z-R-S
      R=A(L+2,L+1)
      S=DABS(P)+DABS(Q)+DABS(R)
      P=P/S
      Q=Q/S
      R=R/S
      CALL QRSTEP (A,V,P,Q,R,L,M,N,NA,NV)
      IF (DABS(A(M-1,M-2)).GT.EPS*(DABS(A(M-1,M-1))+DABS(A(M-2,M-2))))
     +    GO TO 80
      A(M-1,M-2)=0.0D0
      RETURN
C
C     LAST LINE OF EXCHNG
C
      END
      SUBROUTINE QRSTEP (A,V,P,Q,R,NL,NU,N,NA,NV)
C
C     *****PARAMETERS:
      INTEGER N,NA,NL,NU,NV
      DOUBLE PRECISION A(NA,N),P,Q,R,V(NV,N)
C
C     *****LOCAL VARIABLES:
      LOGICAL LAST
      INTEGER I,J,K,NL2,NL3,NUM1
      DOUBLE PRECISION S,X,Y,Z
C
C     *****FUNCTIONS:
      INTEGER MIN0
      DOUBLE PRECISION DABS,DSQRT
C
C     *****SUBROUTINES CALLED:
C     NONE
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     THIS SUBROUTINE PERFORMS ONE IMPLICIT QR STEP ON THE UPPER
C     HESSENBERG MATRIX A. THE SHIFT IS DETERMINED BY THE NUMBERS P,Q,
C     AND R, AND THE STEP IS APPLIED TO ROWS AND COLUMNS NL THROUTH NU.
C     THE TRANSFORMATIONS ARE ACCUMULATED IN THE ARRAY V.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NA,NV            ROW DIMENSIONS OF THE ARRAYS CONTAINING A
C                         AND V, RESPECTIVELY, AS DECLARED IN THE
C                         CALLING PROGRAM DIMENSION STATEMENT;
C
C        A                N X N UPPER HESSENBERG MATRIX ON WHICH THE QR
C                         STEP IS TO BE PERFORMED;
C
C        P,Q,R            PARAMETERS WHICH DETERMINE THE SHIFT;
C
C        NL               THE LOWER LIMIT OF THE STEP;
C
C        NU               THE UPPER LIMIT OF THE STEP;
C
C        N                ORDER OF THE MATRIX A.
C
C     ON OUTPUT:
C
C        V                N X N REAL SCRATCH ARRAY CONTAINING THE
C                         ACCUMULATED TRANSFORMATIONS.
C
C     *****HISTORY:
C              ADAPTED FROM STEWART'S HQR3 SOFTWARE.
C
C     ------------------------------------------------------------------
C
      NL2=NL+2
      DO 10 I=NL2,NU
         A(I,I-2)=0.0D0
10    CONTINUE
      IF (NL2.EQ.NU) GO TO 30
      NL3=NL+3
      DO 20 I=NL3,NU
         A(I,I-3)=0.0D0
20    CONTINUE
30    CONTINUE
      NUM1=NU-1
      DO 130 K=NL,NUM1
         LAST=K.EQ.NUM1
         IF (K.EQ.NL) GO TO 40
         P=A(K,K-1)
         Q=A(K+1,K-1)
         R=0.0D0
         IF (.NOT.LAST) R=A(K+2,K-1)
         X=DABS(P)+DABS(Q)+DABS(R)
         IF (X.EQ.0.0D0) GO TO 130
         P=P/X
         Q=Q/X
         R=R/X
40       CONTINUE
         S=DSQRT(P**2+Q**2+R**2)
         IF (P.LT.0.0D0) S=-S
         IF (K.EQ.NL) GO TO 50
         A(K,K-1)=-S*X
         GO TO 60
50       CONTINUE
         IF (NL.NE.1) A(K,K-1)=-A(K,K-1)
60       CONTINUE
         P=P+S
         X=P/S
         Y=Q/S
         Z=R/S
         Q=Q/P
         R=R/P
         DO 80 J=K,N
            P=A(K,J)+Q*A(K+1,J)
            IF (LAST) GO TO 70
            P=P+R*A(K+2,J)
            A(K+2,J)=A(K+2,J)-P*Z
70          CONTINUE
            A(K+1,J)=A(K+1,J)-P*Y
            A(K,J)=A(K,J)-P*X
80       CONTINUE
         J=MIN0(K+3,NU)
         DO 100 I=1,J
            P=X*A(I,K)+Y*A(I,K+1)
            IF (LAST) GO TO 90
            P=P+Z*A(I,K+2)
            A(I,K+2)=A(I,K+2)-P*R
90          CONTINUE
            A(I,K+1)=A(I,K+1)-P*Q
            A(I,K)=A(I,K)-P
100      CONTINUE
         DO 120 I=1,N
            P=X*V(I,K)+Y*V(I,K+1)
            IF (LAST) GO TO 110
            P=P+Z*V(I,K+2)
            V(I,K+2)=V(I,K+2)-P*R
110         CONTINUE
            V(I,K+1)=V(I,K+1)-P*Q
            V(I,K)=V(I,K)-P
120      CONTINUE
130   CONTINUE
      RETURN
C
C     LAST LINE OF QRSTEP
C
      END
      SUBROUTINE SPLIT (A,V,N,L,E1,E2,NA,NV)
C
C     *****PARAMETERS:
      INTEGER L,N,NA,NV
      DOUBLE PRECISION A(NA,N),V(NV,N),E1,E2
C
C     *****LOCAL VARIABLES:
      INTEGER I,J,L1
      DOUBLE PRECISION P,Q,R,T,U,W,X,Y,Z
C
C     *****FUNCTIONS:
      DOUBLE PRECISION DABS,DSQRT
C
C     ****SUBROUTINES CALLED:
C     NONE
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     GIVEN THE UPPER-HESSENBERG MATRIX A WITH A 2 X 2 BLOCK STARTING AT
C     A(L,L), THIS PROGRAM DETERMINES IF THE CORRESPONDING EIGENVALUES
C     ARE REAL OR COMPLEX. IF THEY ARE REAL, A ROTATION IS DETERMINED
C     THAT REDUCES THE BLOCK TO UPPER-TRIANGULAR FORM WITH THE
C     EIGENVALUE OF LARGEST ABSOLUTE VALUE APPEARING FIRST. THE
C     ROTATION IS ACCUMULATED IN THE ARRAY V.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NA,NV            ROW DIMENSIONS OF THE ARRAYS CONTAINING
C                         A AND V, RESPECTIVELY, AS DECLARED IN THE
C                         CALLING PROGRAM DIMENSION STATEMENT;
C
C        A                THE UPPER HESSENBERG MATRIX WHOSE 2 X 2 BLOCK
C                         IS TO BE SPLIT;
C
C        N                ORDER OF THE MATRIX A;
C
C        L                POSITION OF THE 2 X 2 BLOCK.
C
C     ON OUTPUT:
C
C        V                AN N X N ARRAY CONTAINING THE ACCUMULATED
C                         SPLITTING TRANSFORMATION;
C
C        E1,E2            REAL SCALARS. IF THE EIGENVALUES ARE COMPLEX,
C                         E1 AND E2 CONTAIN THEIR COMMON REAL PART AND
C                         POSITIVE IMAGINARY PART (RESPECTIVELY).
C                         IF THE EIGENVALUES ARE REAL, E1 CONTAINS THE
C                         ONE LARGEST IN ABSOLUTE VALUE AND E2 CONTAINS
C                         THE OTHER ONE.
C
C     *****HISTORY:
C              ADAPTED FROM STEWART'S HQR3 SOFTWARE.
C
C     ------------------------------------------------------------------
C
      X=A(L+1,L+1)
      Y=A(L,L)
      W=A(L,L+1)*A(L+1,L)
      P=(Y-X)/2.0D0
      Q=P**2+W
      IF (Q.GE.0.0D0) GO TO 10
      E1=P+X
      E2=DSQRT(-Q)
      RETURN
10    CONTINUE
      Z=DSQRT(Q)
      IF (P.LT.0.0D0) GO TO 20
      Z=P+Z
      GO TO 30
20    CONTINUE
      Z=P-Z
30    CONTINUE
      IF (Z.EQ.0.0D0) GO TO 40
      R=-W/Z
      GO TO 50
40    CONTINUE
      R=0.0D0
50    CONTINUE
      IF (DABS(X+Z).GE.DABS(X+R)) Z=R
      Y=Y-X-Z
      X=-Z
      T=A(L,L+1)
      U=A(L+1,L)
      IF (DABS(Y)+DABS(U).LE.DABS(T)+DABS(X)) GO TO 60
      Q=U
      P=Y
      GO TO 70
60    CONTINUE
      Q=X
      P=T
70    CONTINUE
      R=DSQRT(P**2+Q**2)
      IF (R.GT.0.0D0) GO TO 80
      E1=A(L,L)
      E2=A(L+1,L+1)
      A(L+1,L)=0.0D0
      RETURN
80    CONTINUE
      P=P/R
      Q=Q/R
      DO 90 J=L,N
         Z=A(L,J)
         A(L,J)=P*Z+Q*A(L+1,J)
         A(L+1,J)=P*A(L+1,J)-Q*Z
90    CONTINUE
      L1=L+1
      DO 100 I=1,L1
         Z=A(I,L)
         A(I,L)=P*Z+Q*A(I,L+1)
         A(I,L+1)=P*A(I,L+1)-Q*Z
100   CONTINUE
      DO 110 I=1,N
         Z=V(I,L)
         V(I,L)=P*Z+Q*V(I,L+1)
         V(I,L+1)=P*V(I,L+1)-Q*Z
110   CONTINUE
      A(L+1,L)=0.0D0
      E1=A(L,L)
      E2=A(L+1,L+1)
      RETURN
C
C     LAST LINE OF SPLIT
C
      END


      SUBROUTINE MLINEQ (NA,NB,N,M,A,B,COND,IPVT,WORK)
C
C     *****PARAMETERS:
      INTEGER NA,NB,N,M,IPVT(N)
      DOUBLE PRECISION A(NA,N),B(NB,M),COND,WORK(N)
C
C     *****LOCAL VARIABLES:
      INTEGER J
C
C     *****SUBROUTINES CALLED:
C     DGECOM,DGESLM
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     THIS SUBROUTINE SOLVES THE MATRIX LINEAR EQUATION
C                             A*X = B
C     WHERE A IS AN N X N (INVERTIBLE) MATRIX AND B IS AN N X M
C     MATRIX.  SUBROUTINE DGECOM IS CALLED ONCE FOR THE LU-FACTOR-
C     IZATION OF A AND SUBROUTINE DGESLM IS CALLED M TIMES FOR
C     FORWARD ELIMINATION AND BACK SUBSTITUTION TO PRODUCE THE
C     M COLUMNS OF THE SOLUTION MATRIX X = (A-INVERSE)*B.  AN
C     ESTIMATE OF THE CONDITION OF A IS RETURNED.
C
C     ***********
C     SHOULD A BE SINGULAR TO WORKING ACCURACY, A MESSAGE TO THAT
C     EFFECT IS PRODUCED.  IT IS ACTUALLY MUCH BETTER FOR THE
C     CALLING PROGRAM TO INTERROGATE THE VALUE OF COND (OR BETTER,
C     RCOND) AND THEN TAKE APPROPRIATE ACTION SUCH AS CALLING A
C     GENERAL ERROR/EXCEPTION HANDLER OR EVEN PRINTING A MESSAGE
C     DIRECTLY.  SUCH MODIFICATION IS EASILY MAKE TO THIS SUBROUTINE.
C     ***********
C
C     *****PARAMETER DESCRIPTION:
C
C     ON INPUT:
C
C        NA,NB            ROW DIMENSIONS OF THE ARRAYS CONTAINING A AND
C                         B, RESPECTIVELY, AS DECLARED IN THE CALLING
C                         PROGRAM DIMENSION STATEMENT;
C
C        N                ORDER OF THE MATRIX A AND NUMBER OF ROWS OF
C                         THE MATRIX B;
C
C        M                NUMBER OF COLUMNS OF THE MATRIX B;
C
C        A                N X N COEFFICIENT MATRIX;
C
C        B                N X M RIGHT HAND SIDE MATRIX.
C
C     ON OUTPUT:
C
C        B                SOLUTION MATRIX  X = (A-INVERSE)*B;
C
C        COND             AN ESTIMATE OF THE CONDITION OF A.
C                         = 1/RCOND  WHERE RCOND IS THE INVERSE
C                         OF THE CONDITION ESTIMATE (SEE THE LINPACK
C                         USER'S GUIDE FOR DETAILS);
C
C        IPVT             PIVOT VECTOR OF LENGTH N (SEE DGECOM
C                         DOCUMENTATION);
C
C        WORK             A REAL SCRATCH VECTOR OF LENGTH N.
C
C     *****APPLICATIONS AND USAGE RESTRICTIONS:
C     (1)THE VALUE OF COND SHOULD ALWAYS BE CHECKED BY THE CALLING
C        PROGRAM.  SHOULD A BE NEAR-SINGULAR (OR SINGULAR TO WORKING
C        ACCURACY) THE DATA SHOULD BE INVESTIGATED FOR POSSIBLE
C        ERRORS.  IF THERE ARE NONE AND THE PROBLEM IS APPARENTLY
C        WELL-POSED AND/OR MEANINGFUL, SINGULAR VALUE ANALYSIS MAY
C        THEN BE A MORE RELIABLE SOLUTION TECHNIQUE (E.G., EISPACK
C        SUBROUTINES  SVD  AND  MINFIT  OR LINPACK SUBROUTINE
C        DSVDC).
C     (2)MLINEQ CAN BE USED TO COMPUTE THE INVERSE OF A:  SIMPLY SOLVE
C        A*X = I WHERE  I  IS THE N X N IDENTITY MATRIX.
C     (3)IF THE SOLUTION TO X*A = B  (X = B*(A-INVERSE)) IS DESIRED,
C        SIMPLY TRANSPOSE THE SOLUTION OF
C         T      T
C        A *X = B .
C
C     *****ALGORITHM NOTES:
C     THE CONTENTS OF A ARE MODIFIED BY THIS SUBROUTINE.  SHOULD THE
C     ORIGINAL COEFFICIENTS OF A BE NEEDED SUBSEQUENTLY, THE
C     CONTENTS OF A SHOULD BE SAVED PRIOR TO THE CALL TO MLINEQ.
C
C     *****HISTORY:
C     WRITTEN BY ALAN J. LAUB (ECE DEPT., UNIV. OF CALIFORNIA,
C     SANTA BARBARA, CA 93106, PH.: (805) 961-3616), MAY 1980.
C     MOST RECENT VERSION:  AUG. 24, 1984.
C
C     ------------------------------------------------------------------
C
      CALL DGECOM (A,NA,N,IPVT,COND,WORK)
      IF ((1.0D0 + COND) .GT. 1.0D0) GO TO 20
      WRITE (6,10)
   10 FORMAT (' MATRIX IS SINGULAR TO WORKING PRECISION')
      RETURN
   20 CONTINUE
      COND = 1.0D0/COND
      DO 30 J=1,M
C
C        COMPUTE  (J-TH COLUMN OF X) = (A-INVERSE)*(J-TH COLUMN OF B)
C
         CALL DGESLM (A,NA,N,IPVT,B(1,J))
   30 CONTINUE
      RETURN
      END
      SUBROUTINE DGECOM (A,LDA,N,IPVT,RCOND,Z)
      INTEGER LDA,N,IPVT(N)
      DOUBLE PRECISION A(LDA,N),Z(N)
      DOUBLE PRECISION RCOND
C
C     DGECOM FACTORS A DOUBLE PRECISION MATRIX BY GAUSSIAN ELIMINATION
C     AND ESTIMATES THE CONDITION OF THE MATRIX.
C
C     IF RCOND IS NOT NEEDED, DGEFAM IS SLIGHTLY FASTER.
C     TO SOLVE  A*X = B , FOLLOW DGECOM BY DGESLM.
C     TO COMPUTE  INVERSE(A)*C , FOLLOW DGECOM BY DGESLM.
C
C     ON ENTRY:
C
C        A       DOUBLE PRECISION(LDA,N)
C                THE MATRIX TO BE FACTORED.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY A AS DECLARED
C                IN THE MAIN CALLING PROGRAM.
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX A.
C
C     ON RETURN:
C
C        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS
C                WHICH WERE USED TO OBTAIN IT.
C                THE FACTORIZATION CAN BE WRITTEN  A = L*U  WHERE
C                L IS A PRODUCT OF PERMUTATION AND UNIT LOWER
C                TRIANGULAR MATRICES AND U IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N)
C                AN INTEGER VECTOR OF PIVOT INDICES.
C                   IPVT(K) = THE INDEX OF THE K-TH PIVOT ROW
C                THE DETERMINANT OF A CAN BE OBTAINED ON OUTPUT BY
C                   DET(A) = S*A(1,1)*A(2,2)* ... *A(N,N)
C                WHERE S = (-1)**(NUMBER OF TIMES IPVT(K) .NE. K)
C                BUT THIS RESULT SHOULD BE USED WITH CAUTION AS IT
C                MAY EASILY UNDERFLOW OR OVERFLOW.
C
C        RCOND   DOUBLE PRECISION
C                AN ESTIMATE OF THE RECIPROCAL CONDITION OF A.
C                FOR THE SYSTEM  A*X = B , RELATIVE PERTURBATIONS
C                IN A AND B OF SIZE EPSILON MAY CAUSE RELATIVE
C                PERTURBATIONS IN X OF SIZE  EPSILON/RCOND.
C                IF RCOND IS SO SMALL THAT THE LOGICAL EXPRESSION
C                             1.0D0 + RCOND  .EQ.  1.0D0
C                IS TRUE, THEN A MAY BE SINGULAR TO WORKING
C                PRECISION.  IN PARTICULAR, RCOND IS ZERO IF
C                EXACT SINGULARITY IS DETECTED OR THE ESTIMATE
C                UNDERFLOWS.
C
C        Z       DOUBLE PRECISION(N)
C                A WORK VECTOR WHOSE CONTENTS ARE USUALLY UNIMPORTANT.
C                IF A IS CLOSE TO A SINGULAR MATRIX, THEN Z IS AN
C                APPROXIMATE NULL VECTOR IN THE SENSE THAT
C                       NORM(A*Z) = RCOND*NORM(A)*NORM(Z).
C
C     THIS VERSION IS ADAPTED FROM THE LINPACK SUBROUTINE DGECO BY
C     REPLACEMENT OF CALLS TO THE BLAS WITH IN-LINE CODE.  MODIFICATION
C     DONE BY ALAN J. LAUB, UNIVERSITY OF SOUTHERN CALIFORNIA,
C     APRIL 1980.
C
C     SUBROUTINES AND FUNCTIONS CALLED:
C
C     DGEFAM
C
C     FORTRAN FUNCTIONS CALLED:
C
      DOUBLE PRECISION DABS,DSIGN
C
C     INTERNAL VARIABLES:
C
      INTEGER I,INFO,J,K,KB,KM1,KP1,L
      DOUBLE PRECISION ANORM,EK,S,SM,T,WK,WKM,YNORM
C
C     COMPUTE 1-NORM OF A
C
      ANORM = 0.0D0
      DO 20 J=1,N
         T = 0.0D0
         DO 10 I=1,N
            T = T+DABS(A(I,J))
   10    CONTINUE
         IF (T .GT. ANORM) ANORM = T
   20 CONTINUE
C
C     FACTOR
C
      CALL DGEFAM(A,LDA,N,IPVT,INFO)
C
C     RCOND = 1/(NORM(A)*(ESTIMATE OF NORM(INVERSE(A)))).
C     ESTIMATE = NORM(Z)/NORM(Y)  WHERE  A*Z = Y AND  TRANS(A)*Y = E.
C     TRANS(A)  IS THE TRANSPOSE OF A.  THE COMPONENTS OF E ARE
C     CHOSEN TO CAUSE MAXIMUM LOCAL GROWTH IN THE ELEMENTS OF W WHERE
C     TRANS(U)*W = E .  THE VECTORS ARE FREQUENTLY RESCALED TO AVOID
C     OVERFLOW.
C
C     SOLVE  TRANS(U)*W = E
C
      EK = 1.0D0
      DO 30 I=1,N
         Z(I) = 0.0D0
   30 CONTINUE
      DO 120 K=1,N
         IF (Z(K) .NE. 0.0D0) EK = DSIGN(EK,-Z(K))
         IF (DABS(EK-Z(K)) .LE. DABS(A(K,K))) GO TO 50
         S = DABS(A(K,K))/DABS(EK-Z(K))
         DO 40 I=1,N
            Z(I) = S*Z(I)
   40    CONTINUE
         EK = S*EK
   50    CONTINUE
         WK = EK-Z(K)
         WKM = -EK-Z(K)
         S = DABS(WK)
         SM = DABS(WKM)
         IF (A(K,K) .EQ. 0.0D0) GO TO 60
         WK = WK/A(K,K)
         WKM = WKM/A(K,K)
         GO TO 70
   60    CONTINUE
         WK = 1.0D0
         WKM = 1.0D0
   70    CONTINUE
         KP1 = K+1
         IF (KP1 .GT. N) GO TO 110
         DO 80 J=KP1,N
            SM = SM+DABS(Z(J)+WKM*A(K,J))
            Z(J) = Z(J)+WK*A(K,J)
            S = S+DABS(Z(J))
   80    CONTINUE
         IF (S .GE. SM) GO TO 100
         T = WKM-WK
         WK = WKM
         DO 90 J=KP1,N
            Z(J) = Z(J)+T*A(K,J)
   90    CONTINUE
  100    CONTINUE
  110    CONTINUE
         Z(K) = WK
  120 CONTINUE
      T = 0.0D0
      DO 130 I=1,N
         T = T+DABS(Z(I))
  130 CONTINUE
      S = 1.0D0/T
      DO 140 I=1,N
         Z(I) = S*Z(I)
  140 CONTINUE
C
C     SOLVE  TRANS(L)*Y = W
C
      DO 190 KB=1,N
         K = N+1-KB
         IF (K .EQ. N) GO TO 160
         KP1 = K+1
         T = 0.0D0
         DO 150 I=KP1,N
            T = T+A(I,K)*Z(I)
  150    CONTINUE
         Z(K) = Z(K)-T
  160    CONTINUE
         IF (DABS(Z(K)) .LE. 1.0D0) GO TO 180
         S = 1.0D0/DABS(Z(K))
         DO 170 I=1,N
            Z(I) = S*Z(I)
  170    CONTINUE
  180    CONTINUE
         L = IPVT(K)
         T = Z(L)
         Z(L) = Z(K)
         Z(K) = T
  190 CONTINUE
      T = 0.0D0
      DO 200 I=1,N
         T = T+DABS(Z(I))
  200 CONTINUE
      S = 1.0D0/T
      DO 210 I=1,N
         Z(I) = S*Z(I)
  210 CONTINUE
C
      YNORM = 1.0D0
C
C     SOLVE  L*V = Y
C
      DO 260 K=1,N
         L = IPVT(K)
         T = Z(L)
         Z(L) = Z(K)
         Z(K) = T
         IF (K .EQ. N) GO TO 230
         KP1 = K+1
         DO 220 I=KP1,N
            Z(I) = Z(I)-T*A(I,K)
  220    CONTINUE
  230    CONTINUE
         IF (DABS(Z(K)) .LE. 1.0D0) GO TO 250
         S = 1.0D0/DABS(Z(K))
         DO 240 I=1,N
            Z(I) = S*Z(I)
  240    CONTINUE
         YNORM = S*YNORM
  250    CONTINUE
  260 CONTINUE
      T = 0.0D0
      DO 270 I=1,N
         T = T+DABS(Z(I))
  270 CONTINUE
      S = 1.0D0/T
      DO 280 I=1,N
         Z(I) = S*Z(I)
  280 CONTINUE
      YNORM = S*YNORM
C
C     SOLVE  U*Z = V
C
      DO 330 KB=1,N
         K = N+1-KB
         IF (DABS(Z(K)) .LE. DABS(A(K,K))) GO TO 300
         S = DABS(A(K,K))/DABS(Z(K))
         DO 290 I=1,N
            Z(I) = S*Z(I)
  290    CONTINUE
         YNORM = S*YNORM
  300    CONTINUE
         IF (A(K,K) .NE. 0.0D0) Z(K) = Z(K)/A(K,K)
         IF (A(K,K) .EQ. 0.0D0) Z(K) = 1.0D0
         T = -Z(K)
         IF (K .EQ. 1) GO TO 320
         KM1 = K-1
         DO 310 I=1,KM1
            Z(I) = Z(I)+T*A(I,K)
  310    CONTINUE
  320    CONTINUE
  330 CONTINUE
C
C     MAKE ZNORM = 1.0D0
C
      T = 0.0D0
      DO 340 I=1,N
         T = T+DABS(Z(I))
  340 CONTINUE
      S = 1.0D0/T
      DO 350 I=1,N
         Z(I) = S*Z(I)
  350 CONTINUE
      YNORM = S*YNORM
C
C     DETERMINE RCOND
C
      IF (ANORM .NE. 0.0D0) RCOND = YNORM/ANORM
      IF (ANORM .EQ. 0.0D0) RCOND = 0.0D0
      RETURN
      END
      SUBROUTINE DGEFAM (A,LDA,N,IPVT,INFO)
      INTEGER LDA,N,IPVT(N),INFO
      DOUBLE PRECISION A(LDA,N)
C
C     DGEFAM FACTORS A DOUBLE PRECISION REAL MATRIX BY
C     GAUSSIAN ELIMINATION.
C
C     DGEFAM IS USUALLY CALLED BY DGECOM, BUT IT CAN BE CALLED
C     DIRECTLY WITH A MODEST SAVING IN TIME IF RCOND IS NOT
C     NEEDED.
C
C     ON ENTRY:
C
C        A       DOUBLE PRECISION(LDA,N)
C                THE MATRIX TO BE FACTORED.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY A AS DECLARED
C                IN THE MAIN CALLING PROGRAM.
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX A.
C
C     ON RETURN:
C
C        A       AN UPPER TRIANGULAR MATRIX AND THE MULTIPLIERS
C                WHICH WERE USED TO OBTAIN IT.
C                THE FACTORIZATION CAN BE WRITTEN  A = L * U WHERE
C                L IS A PRODUCT OF PERMUTATION AND UNIT LOWER
C                TRIANGULAR MATRICES AND U IS UPPER TRIANGULAR.
C
C        IPVT    INTEGER(N)
C                AN INTEGER VECTOR OF PIVOT INDICES.
C                   IPVT(K) = THE INDEX OF THE K-TH PIVOT ROW
C                THE DETERMINANT OF A CAN BE OBTAINED ON OUTPUT BY
C                   DET(A) = S*A(1,1)*A(2,2)* ... *A(N,N)
C                WHERE S = (-1)**(NUMBER OF TIMES IPVT(K) .NE. K)
C                BUT THIS RESULT SHOULD BE USED WITH CAUTION AS IT
C                MAY EASILY UNDERFLOW OR OVERFLOW.
C
C        INFO    INTEGER
C                = 0  NORMAL VALUE.
C                = K  IF  U(K,K) .EQ. 0.0D0.  THIS IS NOT AN ERROR
C                     CONDITION FOR THIS SUBROUTINE, BUT IT DOES
C                     INDICATE THAT DGESLM WILL DIVIDE BY ZERO IF
C                     CALLED.
C                     USE RCOND IN DGECOM FOR A RELIABLE INDICATION OF
C                     SINGULARITY.
C
C     THIS VERSION IS ADAPTED FROM THE LINPACK SUBROUTINE DGEFA BY
C     REPLACEMENT OF CALLS TO THE BLAS WITH IN-LINE CODE.  MODIFICATION
C     DONE BY ALAN J. LAUB, UNIVERSITY OF SOUTHERN CALIFORNIA,
C     APRIL 1980.
C
C     FORTRAN FUNCTIONS CALLED:
C
      DOUBLE PRECISION DABS
C
C     INTERNAL VARIABLES:
C
      INTEGER I,J,K,KP1,L,NM1
      DOUBLE PRECISION T
C
C     GAUSSIAN ELIMINATION WITH PARTIAL PIVOTING
C
      IPVT(N) = 1
      INFO = 0
      NM1 = N-1
      IF (NM1 .LT. 1) GO TO 100
      DO 90 K=1,NM1
         KP1 = K+1
C
C        FIND L = PIVOT INDEX
C
         L = K
         DO 10 I=KP1,N
            IF (DABS(A(I,K)) .GT. DABS(A(L,K))) L = I
   10    CONTINUE
         IPVT(K) = L
C
C        ZERO PIVOT IMPLIES THIS COLUMN ALREADY TRIANGULARIZED
C
         IF (A(L,K) .EQ. 0.0D0) GO TO 70
C
C        INTERCHANGE IF NECESSARY
C
         IF (L .EQ. K) GO TO 20
         T = A(L,K)
         A(L,K) = A(K,K)
         A(K,K) = T
   20    CONTINUE
C
C        COMPUTE MULTIPLIERS
C
         T = 1.0D0/A(K,K)
         DO 30 I=KP1,N
            A(I,K) = T*A(I,K)
   30    CONTINUE
C
C        ROW ELIMINATION WITH COLUMN INDEXING
C
         DO 60 J=KP1,N
            T = A(L,J)
            IF (L .EQ. K) GO TO 40
            A(L,J) = A(K,J)
            A(K,J) = T
   40       CONTINUE
            DO 50 I=KP1,N
               A(I,J) = A(I,J)-T*A(I,K)
   50       CONTINUE
   60    CONTINUE
         GO TO 80
   70    CONTINUE
         INFO = K
   80    CONTINUE
   90 CONTINUE
  100 CONTINUE
      IPVT(N) = N
      IF (A(N,N) .EQ. 0.0D0) INFO = N
      RETURN
      END
      SUBROUTINE DGESLM (A,LDA,N,IPVT,B)
      INTEGER LDA,N,IPVT(N)
      DOUBLE PRECISION A(LDA,N),B(N)
C
C     DGESLM SOLVES THE (REAL) LINEAR SYSTEM
C            A * X = B
C     USING THE FACTORS COMPUTED BY DGECOM OR DGEFAM.
C
C     ON ENTRY:
C
C        A       DOUBLE PRECISION(LDA,N)
C                THE OUTPUT FROM DGECOM OR DGEFAM.
C
C        LDA     INTEGER
C                THE LEADING DIMENSION OF THE ARRAY A AS DECLARED
C                IN THE MAIN CALLING PROGRAM.
C
C        N       INTEGER
C                THE ORDER OF THE MATRIX A.
C
C        IPVT    INTEGER(N)
C                THE PIVOT VECTOR FROM DGECOM OR DGEFAM.
C
C        B       DOUBLE PRECISION(N)
C                THE RIGHT HAND SIDE VECTOR.
C
C     ON RETURN:
C
C        B       THE SOLUTION VECTOR X.
C
C     ERROR CONDITION:
C
C        A DIVISION BY ZERO WILL OCCUR IF THE INPUT FACTOR CONTAINS A
C        ZERO ON THE DIAGONAL.  TECHNICALLY THIS INDICATES SINGULARITY
C        BUT IT IS OFTEN CAUSED BY IMPROPER ARGUMENTS OR IMPROPER
C        SETTING OF LDA.  IT WILL NOT OCCUR IF THE SUBROUTINES ARE
C        CALLED CORRECTLY AND IF DGECOM HAS SET RCOND .GT. 0.0
C        OR DGEFAM HAS SET INFO .EQ. 0.
C
C     TO COMPUTE  INVERSE(A) * B  WHERE B IS A MATRIX WITH M COLUMNS
C             CALL DGECOM (A,LDA,N,IPVT,RCOND,Z)
C             IF (RCOND IS TOO SMALL) GO TO ...
C             DO 10 J=1,M
C                CALL DGESLM (A,LDA,N,IPVT,B(1,J))
C          10 CONTINUE
C
C     THIS VERSION IS ADAPTED FROM THE LINPACK SUBROUTINE DGESL BY
C     REPLACEMENT OF CALLS TO THE BLAS WITH IN-LINE CODE.  MODIFICA-
C     TION DONE BY ALAN J. LAUB, UNIVERSITY OF SOUTHERN CALIFORNIA,
C     APRIL 1980.
C
C     FORTRAN FUNCTIONS CALLED:
C
C     NONE
C
C     INTERNAL VARIABLES:
C
      INTEGER I,K,KB,KM1,KP1,L,NM1
      DOUBLE PRECISION T
C
      IF (N .EQ. 1) GO TO 60
      NM1 = N-1
C
C     FIRST SOLVE L*Y = B
C
      DO 30 K=1,NM1
         KP1 = K+1
         L = IPVT(K)
         T = B(L)
         IF (L .EQ. K) GO TO 10
         B(L) = B(K)
         B(K) = T
   10    CONTINUE
         DO 20 I=KP1,N
            B(I) = B(I)-T*A(I,K)
   20    CONTINUE
   30 CONTINUE
C
C     NOW SOLVE  U*X = Y
C
      DO 50 KB=1,NM1
         K = N+1-KB
         KM1 = K-1
         B(K) = B(K)/A(K,K)
         T = -B(K)
         DO 40 I=1,KM1
            B(I) = B(I)+T*A(I,K)
   40    CONTINUE
   50 CONTINUE
   60 CONTINUE
      B(1) = B(1)/A(1,1)
      RETURN
      END


      SUBROUTINE MULWOA (NA,NB,N,A,B,WORK)
C
C     *****PARAMETERS:
      INTEGER NA,NB,N
      DOUBLE PRECISION A(NA,N),B(NB,N),WORK(N)
C
C     *****LOCAL VARIABLES:
      INTEGER I,J,K
C
C     *****SUBROUTINES CALLED:
C     NONE
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     THIS SUBROUTINE OVERWRITES THE ARRAY A WITH THE MATRIX PRODUCT
C        A*B.  BOTH A AND B ARE N X N ARRAYS AND MUST BE DISTINCT.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NA,NB            ROW DIMENSIONS OF THE ARRAYS CONTAINING A AND
C                         B, RESPECTIVELY, AS DECLARED IN THE CALLING
C                         PROGRAM DIMENSION STATEMENT;
C
C        N                ORDER OF THE MATRICES A AND B;
C
C        A                AN N X N MATRIX;
C
C        B                AN N X N MATRIX.
C
C     ON OUTPUT:
C
C        A                AN N X N ARRAY CONTAINING A*B;
C
C        WORK             A REAL SCRATCH VECTOR OF LENGTH N.
C
C     *****HISTORY:
C     WRITTEN BY ALAN J. LAUB (PRESENTLY AT: ECE DEPT., UNIV. OF
C     CALIFORNIA, SANTA BARBARA, CA 93106; PH.: (805) 961-3616),
C     SEPTEMBER 1977.
C     MOST RECENT VERSION: SEP. 21, 1977.
C
C     ------------------------------------------------------------------
C
      DO 40 I=1,N
         DO 20 J=1,N
            WORK(J)=0.0D0
            DO 10 K=1,N
               WORK(J)=WORK(J)+A(I,K)*B(K,J)
10          CONTINUE
20       CONTINUE
         DO 30 J=1,N
            A(I,J)=WORK(J)
30       CONTINUE
40    CONTINUE
      RETURN
C
C     LAST LINE OF MULWOA
C
      END
      SUBROUTINE MULWOB (NA,NB,N,A,B,WORK)
C
C     *****PARAMETERS:
      INTEGER NA,NB,N
      DOUBLE PRECISION A(NA,N),B(NB,N),WORK(N)
C
C     *****LOCAL VARIABLES:
      INTEGER I,J,K
C
C     *****SUBROUTINES CALLED:
C     NONE
C
C     ------------------------------------------------------------------
C
C     *****PURPOSE:
C     THIS SUBROUTINE OVERWRITES THE ARRAY B WITH THE MATRIX PRODUCT
C        A*B.  BOTH A AND B ARE N X N ARRAYS AND MUST BE DISTINCT.
C
C     *****PARAMETER DESCRIPTION:
C     ON INPUT:
C        NA,NB            ROW DIMENSIONS OF THE ARRAYS CONTAINING A AND
C                         B, RESPECTIVELY, AS DECLARED IN THE CALLING
C                         PROGRAM DIMENSION STATEMENT;
C
C        N                ORDER OF THE MATRICES A AND B;
C
C        A                AN N X N MATRIX;
C
C        B                AN N X N MATRIX.
C
C     ON OUTPUT:
C
C        B                AN N X N ARRAY CONTAINING A*B;
C
C        WORK             A REAL SCRATCH VECTOR OF LENGTH N.
C
C     *****HISTORY:
C     WRITTEN BY ALAN J. LAUB (PRESENTLY AT: ECE DEPT., UNIV. OF
C     CALIFORNIA, SANTA BARBARA, CA 93106; PH.: (805) 961-3616),
C     SEPTEMBER 1977.
C     MOST RECENT VERSION: SEP. 21, 1977.
C
C     ------------------------------------------------------------------
C
      DO 50 J=1,N
         DO 10 I=1,N
            WORK(I)=0.0D0
10       CONTINUE
         DO 30 K=1,N
            DO 20 I=1,N
               WORK(I)=WORK(I)+A(I,K)*B(K,J)
20          CONTINUE
30       CONTINUE
         DO 40 I=1,N
            B(I,J)=WORK(I)
40       CONTINUE
50    CONTINUE
      RETURN
C
C     LAST LINE OF MULWOB
C
      END


      SUBROUTINE BALANC(NM,N,A,LOW,IGH,SCALE)
C
      INTEGER I,J,K,L,M,N,JJ,NM,IGH,LOW,IEXC
      DOUBLE PRECISION A(NM,N),SCALE(N)
      DOUBLE PRECISION C,F,G,R,S,B2,RADIX
      DOUBLE PRECISION DABS
      LOGICAL NOCONV
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALANCE,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE BALANCES A REAL MATRIX AND ISOLATES
C     EIGENVALUES WHENEVER POSSIBLE.
C
C     ON INPUT-
C
C	 NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C	   ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C	   DIMENSION STATEMENT,
C
C	 N IS THE ORDER OF THE MATRIX,
C
C	 A CONTAINS THE INPUT MATRIX TO BE BALANCED.
C
C     ON OUTPUT-
C
C	 A CONTAINS THE BALANCED MATRIX,
C
C	 LOW AND IGH ARE TWO INTEGERS SUCH THAT A(I,J)
C	   IS EQUAL TO ZERO IF
C	    (1) I IS GREATER THAN J AND
C	    (2) J=1,...,LOW-1 OR I=IGH+1,...,N,
C
C	 SCALE CONTAINS INFORMATION DETERMINING THE
C	    PERMUTATIONS AND SCALING FACTORS USED.
C
C     SUPPOSE THAT THE PRINCIPAL SUBMATRIX IN ROWS LOW THROUGH IGH
C     HAS BEEN BALANCED, THAT P(J) DENOTES THE INDEX INTERCHANGED
C     WITH J DURING THE PERMUTATION STEP, AND THAT THE ELEMENTS
C     OF THE DIAGONAL MATRIX USED ARE DENOTED BY D(I,J).  THEN
C	 SCALE(J) = P(J),    FOR J = 1,...,LOW-1
C		  = D(J,J),	 J = LOW,...,IGH
C		  = P(J)	 J = IGH+1,...,N.
C     THE ORDER IN WHICH THE INTERCHANGES ARE MADE IS N TO IGH+1,
C     THEN 1 TO LOW-1.
C
C     NOTE THAT 1 IS RETURNED FOR IGH IF IGH IS ZERO FORMALLY.
C
C     THE ALGOL PROCEDURE EXC CONTAINED IN BALANCE APPEARS IN
C     BALANC  IN LINE.	(NOTE THAT THE ALGOL ROLES OF IDENTIFIERS
C     K,L HAVE BEEN REVERSED.)
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C     ********** RADIX IS A MACHINE DEPENDENT PARAMETER SPECIFYING
C		 THE BASE OF THE MACHINE FLOATING POINT REPRESENTATION.
C
C		 **********
      RADIX = 2.
C
      B2 = RADIX * RADIX
      K = 1
      L = N
      GO TO 100
C     ********** IN-LINE PROCEDURE FOR ROW AND
C		 COLUMN EXCHANGE **********
   20 SCALE(M) = J
      IF (J .EQ. M) GO TO 50
C
      DO 30 I = 1, L
	 F = A(I,J)
	 A(I,J) = A(I,M)
	 A(I,M) = F
   30 CONTINUE
C
      DO 40 I = K, N
	 F = A(J,I)
	 A(J,I) = A(M,I)
	 A(M,I) = F
   40 CONTINUE
C
   50 GO TO (80,130), IEXC
C     ********** SEARCH FOR ROWS ISOLATING AN EIGENVALUE
C		 AND PUSH THEM DOWN **********
   80 IF (L .EQ. 1) GO TO 280
      L = L - 1
C     ********** FOR J=L STEP -1 UNTIL 1 DO -- **********
  100 DO 120 JJ = 1, L
	 J = L + 1 - JJ
C
	 DO 110 I = 1, L
	    IF (I .EQ. J) GO TO 110
	    IF (A(J,I) .NE. 0.0D0) GO TO 120
  110	 CONTINUE
C
	 M = L
	 IEXC = 1
	 GO TO 20
  120 CONTINUE
C
      GO TO 140
C     ********** SEARCH FOR COLUMNS ISOLATING AN EIGENVALUE
C		 AND PUSH THEM LEFT **********
  130 K = K + 1
C
  140 DO 170 J = K, L
C
	 DO 150 I = K, L
	    IF (I .EQ. J) GO TO 150
	    IF (A(I,J) .NE. 0.0D0) GO TO 170
  150	 CONTINUE
C
	 M = K
	 IEXC = 2
	 GO TO 20
  170 CONTINUE
C     ********** NOW BALANCE THE SUBMATRIX IN ROWS K TO L **********
      DO 180 I = K, L
  180 SCALE(I) = 1.0D0
C     ********** ITERATIVE LOOP FOR NORM REDUCTION **********
  190 NOCONV = .FALSE.
C
      DO 270 I = K, L
	 C = 0.0D0
	 R = 0.0D0
C
	 DO 200 J = K, L
	    IF (J .EQ. I) GO TO 200
	    C = C + DABS(A(J,I))
	    R = R + DABS(A(I,J))
  200	 CONTINUE
C     ********** GUARD AGAINST ZERO C OR R DUE TO UNDERFLOW **********
	 IF (C .EQ. 0.0D0 .OR. R .EQ. 0.0D0) GO TO 270
	 G = R / RADIX
	 F = 1.0D0
	 S = C + R
  210	 IF (C .GE. G) GO TO 220
	 F = F * RADIX
	 C = C * B2
	 GO TO 210
  220	 G = R * RADIX
  230	 IF (C .LT. G) GO TO 240
	 F = F / RADIX
	 C = C / B2
	 GO TO 230
C     ********** NOW BALANCE **********
  240	 IF ((C + R) / F .GE. 0.95 * S) GO TO 270
	 G = 1.0D0 / F
	 SCALE(I) = SCALE(I) * F
	 NOCONV = .TRUE.
C
	 DO 250 J = K, N
  250	 A(I,J) = A(I,J) * G
C
	 DO 260 J = 1, L
  260	 A(J,I) = A(J,I) * F
C
  270 CONTINUE
C
      IF (NOCONV) GO TO 190
C
  280 LOW = K
      IGH = L
      RETURN
C     ********** LAST CARD OF BALANC **********
      END
      SUBROUTINE BALBAK(NM,N,LOW,IGH,SCALE,M,Z)
C
      INTEGER I,J,K,M,N,II,NM,IGH,LOW
      DOUBLE PRECISION SCALE(N),Z(NM,M)
      DOUBLE PRECISION S
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALBAK,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE FORMS THE EIGENVECTORS OF A REAL GENERAL
C     MATRIX BY BACK TRANSFORMING THOSE OF THE CORRESPONDING
C     BALANCED MATRIX DETERMINED BY  BALANC.
C
C     ON INPUT-
C
C	 NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C	   ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C	   DIMENSION STATEMENT,
C
C	 N IS THE ORDER OF THE MATRIX,
C
C	 LOW AND IGH ARE INTEGERS DETERMINED BY  BALANC,
C
C	 SCALE CONTAINS INFORMATION DETERMINING THE PERMUTATIONS
C	   AND SCALING FACTORS USED BY	BALANC,
C
C	 M IS THE NUMBER OF COLUMNS OF Z TO BE BACK TRANSFORMED,
C
C	 Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE EIGEN-
C	   VECTORS TO BE BACK TRANSFORMED IN ITS FIRST M COLUMNS.
C
C     ON OUTPUT-
C
C	 Z CONTAINS THE REAL AND IMAGINARY PARTS OF THE
C	   TRANSFORMED EIGENVECTORS IN ITS FIRST M COLUMNS.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
      IF (M .EQ. 0) GO TO 200
      IF (IGH .EQ. LOW) GO TO 120
C
      DO 110 I = LOW, IGH
	 S = SCALE(I)
C     ********** LEFT HAND EIGENVECTORS ARE BACK TRANSFORMED
C		 IF THE FOREGOING STATEMENT IS REPLACED BY
C		 S=1.0D0/SCALE(I). **********
	 DO 100 J = 1, M
  100	 Z(I,J) = Z(I,J) * S
C
  110 CONTINUE
C     ********- FOR I=LOW-1 STEP -1 UNTIL 1,
C		IGH+1 STEP 1 UNTIL N DO -- **********
  120 DO 140 II = 1, N
	 I = II
	 IF (I .GE. LOW .AND. I .LE. IGH) GO TO 140
	 IF (I .LT. LOW) I = LOW - II
	 K = SCALE(I)
	 IF (K .EQ. I) GO TO 140
C
	 DO 130 J = 1, M
	    S = Z(I,J)
	    Z(I,J) = Z(K,J)
	    Z(K,J) = S
  130	 CONTINUE
C
  140 CONTINUE
C
  200 RETURN
C     ********** LAST CARD OF BALBAK **********
      END
      SUBROUTINE ORTHES(NM,N,LOW,IGH,A,ORT)
C
      INTEGER I,J,M,N,II,JJ,LA,MP,NM,IGH,KP1,LOW
      DOUBLE PRECISION A(NM,N),ORT(IGH)
      DOUBLE PRECISION F,G,H,SCALE
      DOUBLE PRECISION DSQRT,DABS,DSIGN
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ORTHES,
C     NUM. MATH. 12, 349-368(1968) BY MARTIN AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 339-358(1971).
C
C     GIVEN A REAL GENERAL MATRIX, THIS SUBROUTINE
C     REDUCES A SUBMATRIX SITUATED IN ROWS AND COLUMNS
C     LOW THROUGH IGH TO UPPER HESSENBERG FORM BY
C     ORTHOGONAL SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT-
C
C	 NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C	   ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C	   DIMENSION STATEMENT,
C
C	 N IS THE ORDER OF THE MATRIX,
C
C	 LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C	   SUBROUTINE  BALANC.	IF  BALANC  HAS NOT BEEN USED,
C	   SET LOW=1, IGH=N,
C
C	 A CONTAINS THE INPUT MATRIX.
C
C     ON OUTPUT-
C
C	 A CONTAINS THE HESSENBERG MATRIX.  INFORMATION ABOUT
C	   THE ORTHOGONAL TRANSFORMATIONS USED IN THE REDUCTION
C	   IS STORED IN THE REMAINING TRIANGLE UNDER THE
C	   HESSENBERG MATRIX,
C
C	 ORT CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C	   ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
      LA = IGH - 1
      KP1 = LOW + 1
      IF (LA .LT. KP1) GO TO 200
C
      DO 180 M = KP1, LA
	 H = 0.0D0
	 ORT(M) = 0.0D0
	 SCALE = 0.0D0
C     ********** SCALE COLUMN (ALGOL TOL THEN NOT NEEDED) **********
	 DO 90 I = M, IGH
   90	 SCALE = SCALE + DABS(A(I,M-1))
C
	 IF (SCALE .EQ. 0.0D0) GO TO 180
	 MP = M + IGH
C     ********** FOR I=IGH STEP -1 UNTIL M DO -- **********
	 DO 100 II = M, IGH
	    I = MP - II
	    ORT(I) = A(I,M-1) / SCALE
	    H = H + ORT(I) * ORT(I)
  100	 CONTINUE
C
	 G = -DSIGN(DSQRT(H),ORT(M))
	 H = H - ORT(M) * G
	 ORT(M) = ORT(M) - G
C     ********** FORM (I-(U*UT)/H) * A **********
	 DO 130 J = M, N
	    F = 0.0D0
C     ********** FOR I=IGH STEP -1 UNTIL M DO -- **********
	    DO 110 II = M, IGH
	       I = MP - II
	       F = F + ORT(I) * A(I,J)
  110	    CONTINUE
C
	    F = F / H
C
	    DO 120 I = M, IGH
  120	    A(I,J) = A(I,J) - F * ORT(I)
C
  130	 CONTINUE
C     ********** FORM (I-(U*UT)/H)*A*(I-(U*UT)/H) **********
	 DO 160 I = 1, IGH
	    F = 0.0D0
C     ********** FOR J=IGH STEP -1 UNTIL M DO -- **********
	    DO 140 JJ = M, IGH
	       J = MP - JJ
	       F = F + ORT(J) * A(I,J)
  140	    CONTINUE
C
	    F = F / H
C
	    DO 150 J = M, IGH
  150	    A(I,J) = A(I,J) - F * ORT(J)
C
  160	 CONTINUE
C
	 ORT(M) = SCALE * ORT(M)
	 A(M,M-1) = SCALE * G
  180 CONTINUE
C
  200 RETURN
C     ********** LAST CARD OF ORTHES **********
      END
      SUBROUTINE ORTRAN(NM,N,LOW,IGH,A,ORT,Z)
C
      INTEGER I,J,N,KL,MM,MP,NM,IGH,LOW,MP1
      DOUBLE PRECISION A(NM,IGH),ORT(IGH),Z(NM,N)
      DOUBLE PRECISION G
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ORTRANS,
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C
C     THIS SUBROUTINE ACCUMULATES THE ORTHOGONAL SIMILARITY
C     TRANSFORMATIONS USED IN THE REDUCTION OF A REAL GENERAL
C     MATRIX TO UPPER HESSENBERG FORM BY  ORTHES.
C
C     ON INPUT-
C
C	 NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C	   ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C	   DIMENSION STATEMENT,
C
C	 N IS THE ORDER OF THE MATRIX,
C
C	 LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C	   SUBROUTINE  BALANC.	IF  BALANC  HAS NOT BEEN USED,
C	   SET LOW=1, IGH=N,
C
C	 A CONTAINS INFORMATION ABOUT THE ORTHOGONAL TRANS-
C	   FORMATIONS USED IN THE REDUCTION BY	ORTHES
C	   IN ITS STRICT LOWER TRIANGLE,
C
C	 ORT CONTAINS FURTHER INFORMATION ABOUT THE TRANS-
C	   FORMATIONS USED IN THE REDUCTION BY	ORTHES.
C	   ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     ON OUTPUT-
C
C	 Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C	   REDUCTION BY  ORTHES,
C
C	 ORT HAS BEEN ALTERED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C     ********** INITIALIZE Z TO IDENTITY MATRIX **********
      DO 80 I = 1, N
C
	 DO 60 J = 1, N
   60	 Z(I,J) = 0.0D0
C
	 Z(I,I) = 1.0D0
   80 CONTINUE
C
      KL = IGH - LOW - 1
      IF (KL .LT. 1) GO TO 200
C     ********** FOR MP=IGH-1 STEP -1 UNTIL LOW+1 DO -- **********
      DO 140 MM = 1, KL
	 MP = IGH - MM
	 IF (A(MP,MP-1) .EQ. 0.0D0) GO TO 140
	 MP1 = MP + 1
C
	 DO 100 I = MP1, IGH
  100	 ORT(I) = A(I,MP-1)
C
	 DO 130 J = MP, IGH
	    G = 0.0D0
C
	    DO 110 I = MP, IGH
  110	    G = G + ORT(I) * Z(I,J)
C     ********** DIVISOR BELOW IS NEGATIVE OF H FORMED IN ORTHES.
C		 DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW **********
	    G = (G / ORT(MP)) / A(MP,MP-1)
C
	    DO 120 I = MP, IGH
  120	    Z(I,J) = Z(I,J) + G * ORT(I)
C
  130	 CONTINUE
C
  140 CONTINUE
C
  200 RETURN
C     ********** LAST CARD OF ORTRAN **********
      END

